package org.luyinbros.widget;

import android.content.Context;
import android.database.Observable;
import android.graphics.Canvas;
import android.graphics.Rect;
import android.support.annotation.IntDef;
import android.support.annotation.NonNull;
import android.support.annotation.RestrictTo;
import android.util.AttributeSet;
import android.view.View;
import android.view.ViewGroup;

import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.util.ArrayList;
import java.util.List;

import static android.support.annotation.RestrictTo.Scope.LIBRARY_GROUP;

public class ViewManageLayout extends ViewGroup {
    @RestrictTo(LIBRARY_GROUP)
    @IntDef({HORIZONTAL, VERTICAL})
    @Retention(RetentionPolicy.SOURCE)
    public @interface Orientation {
    }

    public static final int HORIZONTAL = 0;
    public static final int VERTICAL = 1;

    private static final String TAG = "ViewManageLayout";
    private LayoutManager mLayoutManager;
    private Adapter mAdapter;
    private List<ViewHolder> views = new ArrayList<>();
    private DataObserver mDataObserver;
    private Rect decorationRect = new Rect();
    private ItemDecoration itemDecoration;

    public ViewManageLayout(Context context) {
        this(context, null);
    }

    public ViewManageLayout(Context context, AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public ViewManageLayout(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
    }

    public void setItemDecoration(ItemDecoration itemDecoration) {
        if (this.itemDecoration == null) {
            setWillNotDraw(false);
        }
        this.itemDecoration = itemDecoration;
        requestLayout();
    }


    public void setAdapter(Adapter adapter) {
        if (mAdapter != null) {
            mAdapter.unregisterDataSetObserver(mDataObserver);
        }
        if (adapter == null) {
            return;
        }
        mAdapter = adapter;
        mDataObserver = new DataObserver();
        mAdapter.registerDataSetObserver(mDataObserver);
        mAdapter.notifyDataSetChanged();
    }

    public void setLayoutManager(@NonNull LayoutManager layoutManager) {
        this.mLayoutManager = layoutManager;
        mLayoutManager.mParent = this;
        requestLayout();
    }

    public LayoutManager getLayoutManager() {
        return mLayoutManager;
    }

    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        if (mLayoutManager != null) {
            mLayoutManager.onMeasure(widthMeasureSpec, heightMeasureSpec);
        } else {
            super.onMeasure(widthMeasureSpec, heightMeasureSpec);
        }
    }

    @Override
    protected void onLayout(boolean changed, int l, int t, int r, int b) {
        if (mLayoutManager != null) {
            mLayoutManager.onLayout(changed, l, t, r, b);
        }
    }

    private Rect getDecorationRect(int position) {
        if (itemDecoration != null) {
            decorationRect.bottom = 0;
            decorationRect.left = 0;
            decorationRect.top = 0;
            decorationRect.right = 0;
            itemDecoration.getItemOffsets(decorationRect, position, this);
            return decorationRect;
        } else {
            decorationRect.bottom = 0;
            decorationRect.left = 0;
            decorationRect.top = 0;
            decorationRect.right = 0;
            return decorationRect;
        }
    }

    @Override
    public LayoutParams generateLayoutParams(AttributeSet attrs) {
        return new LayoutParams(getContext(), attrs);
    }

    @Override
    protected LayoutParams generateLayoutParams(ViewGroup.LayoutParams p) {
        if (p instanceof MarginLayoutParams) {
            return new LayoutParams((MarginLayoutParams) p);
        } else {
            return new LayoutParams(p);
        }
    }

    @Override
    protected LayoutParams generateDefaultLayoutParams() {
        return new LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT);
    }


    @Override
    protected void onDraw(Canvas canvas) {
        if (itemDecoration != null) {
            itemDecoration.onDraw(canvas, this);
        }
    }

    @Override
    public void draw(Canvas canvas) {
        super.draw(canvas);
        if (itemDecoration != null) {
            itemDecoration.onDrawOver(canvas, this);
        }
    }

    @Override
    protected boolean checkLayoutParams(ViewGroup.LayoutParams p) {
        return super.checkLayoutParams(p) && p instanceof LayoutParams;
    }


    public static abstract class Adapter<VH extends ViewHolder> {
        private final AdapterDataObservable mObservable = new AdapterDataObservable();

        public abstract VH onCreateHolder(ViewGroup container, int viewType);

        public abstract void onBindViewHolder(VH holder, int position);

        public int getItemViewType(int position) {
            return 0;
        }

        public abstract int getItemCount();


        public void registerDataSetObserver(AdapterDataObserver observer) {
            mObservable.registerObserver(observer);
        }

        public void unregisterDataSetObserver(AdapterDataObserver observer) {
            mObservable.unregisterObserver(observer);
        }

        public final void notifyDataSetChanged() {
            mObservable.notifyChanged();
        }

        public final void notifyItemChanged(int position) {
            mObservable.notifyItemRangeChanged(position, 1);
        }

        public final void notifyItemRangeChanged(int positionStart, int itemCount) {
            mObservable.notifyItemRangeChanged(positionStart, itemCount);
        }

        public final void notifyItemInserted(int position) {
            mObservable.notifyItemRangeInserted(position, 1);
        }

        public final void notifyItemRangeInserted(int positionStart, int itemCount) {
            mObservable.notifyItemRangeInserted(positionStart, itemCount);
        }

        public final void notifyItemRemoved(int position) {
            mObservable.notifyItemRangeRemoved(position, 1);
        }

        public final void notifyItemRangeRemoved(int positionStart, int itemCount) {
            mObservable.notifyItemRangeRemoved(positionStart, itemCount);
        }
    }

    public class AdapterDataObserver {

        public void onChanged() {
            // Do nothing
        }

        public void onItemRangeChanged(int positionStart, int itemCount) {
            // do nothing
        }

        public void onItemRangeInserted(int positionStart, int itemCount) {
            // do nothing
        }

        public void onItemRangeRemoved(int positionStart, int itemCount) {
            // do nothing
        }
    }

    public static class AdapterDataObservable extends Observable<AdapterDataObserver> {

        public void notifyChanged() {
            for (int i = mObservers.size() - 1; i >= 0; i--) {
                mObservers.get(i).onChanged();
            }
        }

        public void notifyItemRangeChanged(int positionStart, int itemCount) {
            for (int i = mObservers.size() - 1; i >= 0; i--) {
                mObservers.get(i).onItemRangeChanged(positionStart, itemCount);
            }
        }

        public void notifyItemRangeInserted(int positionStart, int itemCount) {
            for (int i = mObservers.size() - 1; i >= 0; i--) {
                mObservers.get(i).onItemRangeInserted(positionStart, itemCount);
            }
        }

        public void notifyItemRangeRemoved(int positionStart, int itemCount) {

            for (int i = mObservers.size() - 1; i >= 0; i--) {
                mObservers.get(i).onItemRangeRemoved(positionStart, itemCount);
            }
        }

    }

    public static class ViewHolder {
        public View itemView;
        int mViewType = 0;
        ViewManageLayout viewManageLayout;

        public ViewHolder(View itemView) {
            this.itemView = itemView;
        }

        public int getAdapterPosition() {
            return viewManageLayout.views.indexOf(this);
        }
    }

    class DataObserver extends AdapterDataObserver {

        @SuppressWarnings("unchecked")
        @Override
        public void onChanged() {
            removeAllViews();
            invalidate();
            views.clear();
            if (mAdapter == null || mAdapter.getItemCount() == 0) {
                return;
            }
            onItemRangeInserted(0, mAdapter.getItemCount());

        }

        @SuppressWarnings("unchecked")
        @Override
        public void onItemRangeChanged(int positionStart, int itemCount) {
            onItemRangeRemoved(positionStart, views.size() - positionStart + 1);
            onItemRangeInserted(positionStart, mAdapter.getItemCount() - positionStart + 1);
        }

        @SuppressWarnings("unchecked")
        @Override
        public void onItemRangeInserted(int positionStart, int itemCount) {
            for (int index = positionStart; index < positionStart + itemCount; index++) {
                ViewHolder viewHolder = mAdapter.onCreateHolder(ViewManageLayout.this, mAdapter.getItemViewType(index));
                if (viewHolder != null) {

                    viewHolder.viewManageLayout = ViewManageLayout.this;
                    mAdapter.onBindViewHolder(viewHolder, index);
                    views.add(index, viewHolder);
                    addView(viewHolder.itemView, index);
                }
            }

        }

        @Override
        public void onItemRangeRemoved(int positionStart, int itemCount) {
            views.subList(positionStart, positionStart + itemCount).clear();
            removeViews(positionStart, itemCount);
            requestLayout();
        }

    }

    public static class ItemDecoration {

        public void onDrawOver(Canvas c, ViewManageLayout parent) {

        }

        public void getItemOffsets(Rect outRect, int itemPosition, ViewManageLayout parent) {
            outRect.set(0, 0, 0, 0);
        }

        public void onDraw(Canvas c, ViewManageLayout parent) {

        }
    }



    public static class LayoutParams extends MarginLayoutParams {


        public LayoutParams(Context c, AttributeSet attrs) {
            super(c, attrs);
        }

        public LayoutParams(int width, int height) {
            super(width, height);
        }

        public LayoutParams(MarginLayoutParams source) {
            super(source);
        }

        public LayoutParams(ViewGroup.LayoutParams source) {
            super(source);
        }
    }


    public abstract static class LayoutManager {
        ViewManageLayout mParent;

        protected abstract void onMeasure(int widthMeasureSpec, int heightMeasureSpec);

        protected abstract void onLayout(boolean changed, int l, int t, int r, int b);

        public int getChildCount() {
            return mParent != null ? mParent.getChildCount() : 0;

        }

        public View getChildAt(int index) {
            return mParent != null ? mParent.getChildAt(index) : null;

        }

        protected final void setMeasuredDimension(int measuredWidth, int measuredHeight) {
            if (mParent != null) {
                mParent.setMeasuredDimension(measuredWidth, measuredHeight);
            }

        }

        protected void measureChild(View child, int parentWidthMeasureSpec,
                                    int parentHeightMeasureSpec) {
            if (mParent != null) {
                mParent.measureChild(child, parentWidthMeasureSpec, parentHeightMeasureSpec);
            }

        }

        protected int getSuggestedMinimumHeight() {
            return mParent != null ? mParent.getSuggestedMinimumHeight() : 0;
        }


        public int getSuggestedMinimumWidth() {
            return mParent != null ? mParent.getSuggestedMinimumWidth() : 0;
        }

        public void requestLayout() {
            if (mParent != null) {
                mParent.requestLayout();
            }

        }

        protected Rect getDecorationRect(int position) {
            return mParent != null ? mParent.getDecorationRect(position) : new Rect();
        }

        protected int getParentMeasuredHeight() {
            return mParent != null ? mParent.getMeasuredHeight() : 0;
        }

        protected int getParentMeasuredWidth() {
            return mParent != null ? mParent.getMeasuredWidth() : 0;
        }

        protected ViewGroup.LayoutParams getLayoutParams() {
            return mParent != null ? mParent.getLayoutParams() : null;
        }
    }
}
